from .compression import compression_ratio
from .patterns.token import token_patterns
from .patterns.part_of_speech import pos_patterns, get_pos
from .utils.memoize import memoized
from .homogenization import homogenization_score
from .ngram_diversity import ngram_diversity_score
from .eigen_auc import eigen_auc
